#!/usr/bin/perl -T
# $Id: ajax_model.cgi,v 1.77 2014/12/12 12:32:23 sofc Exp $
use Cwd;
use strict;
use warnings;
use Impedance::Model;
use Impedance::Header;
use Impedance::IMPCGI qw(:DEFAULT :html :cgi);
use CGI qw(:standard :netscape);
use File::stat;
use File::Copy;
use LATEX;
use SocketClient;
$LATEX::CHEAD = 'Elchemea Analytical';
$LATEX::LFOOT = '';
$LATEX::CFOOT = '\thepage\ of \pageref{LastPage}';
$LATEX::RHEAD = 'Version '.$Impedance::Header::VERSION;
print header();
unless (param('ajax') && param('action'))
{
    print "Oops, no parameters!\n";
    exit;
}

my %slabels = (-3=>'mOhm',0=>'Ohm',3=>'kOhm',6=>'MOhm');
my %slabelseps = (-3=>'m{/Symbol W}',0=>'{/Symbol W}',3=>'k{/Symbol W}',6=>'M{/Symbol W}');
my @adv_opt = ();
my $admode = param('admode');
my $xmin = get_CGI_value_clean('xmin');
my $xmax = get_CGI_value_clean('xmax');
my $ymin = get_CGI_value_clean('ymin');
my $ymax = get_CGI_value_clean('ymax');
foreach ($xmin,$ymin,$xmax,$ymax)
{
    $_ = '' unless defined($_);
}
push @adv_opt, "set xrange [$xmin:$xmax]";
push @adv_opt, "set yrange [$ymin:$ymax]";
push @adv_opt, param('gp_text');
my $scale = get_CGI_value_clean('scale_factor');
$scale = 0 unless ($scale);
my $scalefn = 10**(-$scale);

my $ac = param('action');
if ($ac eq 'new_element')
{	
    my $m = Impedance::Model->new();
    my $type = get_CGI_value_clean('type');
    my $id = $m->add_element($type);
    my @tags = $id->tags();
    print '<table border=0 cellspacing=10 head>';
    print '<caption>'.$id->description().'</caption>';
    foreach my $t (@tags)
    {
	print '<tr><th width=50px>';
	print $t;
	print '</th><td width=50px>';
	print textfield(-id=>'model_'.$t,-name=>$t,-value=>$id->value($t),-force=>1,-size=>6);
	print '</td><td>';
	### Print help button ###
	my $string = $id->description($t);
	$string =~ s/\'/\\\"/g;  ## Fix quotes ##
	print button(-name=>'dummy_desc_'.$t,-value=>'?',-onclick=>"alert('$string')");
	print '</td></tr>';
    }
    print '<tr><td colspan=2>';
    foreach (@tags)
    {
	$_ = '"'.$_.'"';
    }
    print button(-id=>'action_btn',-name=>'action',-value=>'Add element',-class=>'s3',-onclick=>'set_model('.join(',',@tags).')');
    print '<td></tr>';
    my $mode = param('mode');
    $mode = '' unless ($mode);
    if ($id->helpfunction() && $mode !~ /simul/i)
    {
	print '<tr><td colspan=2>';
	print button(-name=>'find_vals',-value=>'Find values',-class=>'s4',-onclick=>"find_values('$type')");
	print '<td></tr>';
    }
    print '</tr></table>';
    exit;
}
if ($ac eq 'fit')
{
    &fit('png');
    exit;
}
if ($ac eq 'fitlist')
{
    my $m = Impedance::IMPCGI::format_model(param('model'));
    my $model1 = Impedance::Model->new();
    $model1->parse($m);
    print join(',',$model1->fit());
    exit;
}
if ($ac eq 'fit_pdf')
{
    my ($fitfile,$errorfile,$txt,$maxf,$fval,$pscs) = &fit('eps');
    my $bodefile = $fitfile;
    $bodefile =~ s/model/bode/ if ($bodefile);
    my @maxfs = @{$maxf};
    my @fvals = @{$fval};
    my @pscs = @{$pscs};
    chdir "/tmp/";
    $LATEX::utf8 = 1;
    my $latex = SemaforeFile->new('/tmp/Fit_report_'.$$.'.tex');
    my $tit = param('filename');
    if ($tit)
    {
	my $s = $tit;
	$s =~ s/_/\\_/g;
	$LATEX::LFOOT = $s 
    }
    $tit = 'Fit report' unless ($tit);
    $latex->append(LATEX::latex_header_simple($tit,'Elchemea Analytical'));
    my $model1 = Impedance::Model->new();
    $model1->parse(param('model'));
    $model1->set_admittance_mode($admode) if ($admode);
    my $tm = $model1->model_text();
    $tm =~ s/\_(\d+)/\_\{$1\}/g;
    $latex->append("Impedance model: \$$tm\$\n\n");
    $txt =~ /\#datapoints\s+\=\s+(\d+)/;
    $latex->append("\n\nNumber of datapoints: $1\n\n");
    $latex->append(start_table('Final values for the model elements','c|c'));
    $latex->append(hline());
    $latex->append(row('Model element','Final value'));
    $latex->append(hline());
    my %f_sort = ();
    foreach my $i (@fvals)
    {
	my ($n,$val) = split(/:/,$i);
	if ($n && $n =~ /_(\d+)/)
	{
	    $f_sort{$i} = $1;
	}
    }
    foreach my $i (sort {$f_sort{$a} <=> $f_sort{$b}} keys %f_sort)
    {
	$latex->append(row(split(/:/,$i)));
    }
    $latex->append(hline());
    my $rs = $model1->F(1e100)->Re();
    my $rp = $model1->F(1e-100)->Re()-$rs;
    $latex->append(row('Rs',sprintf("%.4g (Real part)",$rs)));
    $latex->append(row('Rp',sprintf("%.4g (Real part)",$rp)));
    $latex->append(hline());
    $latex->append(&end_table());
    $latex->append("Rs is determined as the real part of the impedance at 1e100 Hz and Rp is determined as the real part of the impedance at 1e-100 Hz subtracted the Rs value.");
    if (@maxfs)
    {
	$latex->append(start_table('Summit frequencies for the appropriate model elements','c|c'));
	$latex->append(hline());
	$latex->append(row('Model element','Summit frequency (Hz)'));
	$latex->append(hline());
	foreach my $i (@maxfs)
	{
	    $latex->append(row(split(/:/,$i)));
	}
	$latex->append(hline());
	$latex->append(&end_table());
    }
    if (@pscs)
    {
	$latex->append(start_table('Pseudo capacitance for appropriate model elements ($C^* = \frac{(R * Y)^{\frac{1}{n}}}{R}$)','c|c'));
	$latex->append(hline());
	$latex->append(row('Model element','Pseudo capacitance (F)'));
	$latex->append(hline());
	foreach my $i (@pscs)
	{
	    $latex->append(row(split(/:/,$i)));
	}
	$latex->append(hline());
	$latex->append(&end_table());
    }
    $latex->append('\newpage');
    $txt =~ s/^.+Iteration/Iteration/s;
    $latex->append('\begin{verbatim}');
    $txt =~ s/Final set/\n\\end{verbatim}\\newpage\\begin{verbatim}\nFinal set/;
    $txt =~ s/(correlation matrix[\s\w]+:)/$1\\end{verbatim}\n\\begin{tiny}\n\\begin{verbatim}/;
    $latex->append($txt);
    $latex->append('\end{verbatim}','\end{tiny}');
    $latex->append('\newpage');
    $latex->append(&LATEX::figure($fitfile,'Impedance plot of the fit result','fit_file'));
    $latex->append(&LATEX::figure($errorfile,'Error plot for fit','err_file'));
    $latex->append(&LATEX::figure($bodefile,'Bode plot of the fit result','bode_file'));
    $latex->append(&LATEX::figure($errorfile,'Error plot for fit','err_file'));
    $latex->append(end_document());
    my $pdf = SocketClient::socket_client('localhost',4050,'compile',$latex->filename);
    if ($pdf =~ /Error/i)
    {
	print "$pdf\n";
    }
    unlink $latex->filename if ($latex->exist);
    print "$pdf\n";
    exit;
}

if ($ac eq 'delete_point')
{
    my $model1 = Impedance::Model->new();
    $model1->data(param('data'));
    print "OK\n";
    print $model1->delete_point(param('f')),"\n";
    exit;
}

if ($ac eq 'select_points_delete')
{
    my $model = Impedance::Model->new();
    $model->data(param('data'));
    $model->scale_factor($scalefn);
    $model->set_xlabel("Z_real(".$slabels{$scale}.")");
    $model->set_ylabel("-Z_imag(".$slabels{$scale}.")");
    $model->set_ylabel_bode("-Z(".$slabels{$scale}.")");
    my $f = param('f');
    my @data = split(/\n/,$model->data);
    my $plotmode = param('plotmode');
    $plotmode = 'Nyquist' unless ($plotmode);
    my $maxf = 0;
    my $minf = 1e100;
    my $x = 0;
    foreach (@data)
    {
	if (/([\d\.eE\+\-]+)/)
	{
	    $maxf = $1 if ($1 > $maxf);
	    $minf = $1 if ($1 < $minf);
	    $x++;
	}
    }
    my $file = SemaforeFile->new('/tmp/delete_'.$$.'.gp');
    my $file2 = SemaforeFile->new('/tmp/bode_'.$$.'.gp');
    my $pngfn = $pngdir.'delete_'.$$.'.png';
    my $pngb = $pngdir.'bode_delete_'.$$.'.png';
    my $bgcolor = get_CGI_value_clean('bgcolor');
    if ($bgcolor && $bgcolor =~ /white/i)
    {
	$file->writeline($Impedance::Header::gpheader_white);	
	$file2->writeline($Impedance::Header::gpheader_white);	
    }
    else
    {
	$file->writeline($Impedance::Header::gpheader);	
	$file2->writeline($Impedance::Header::gpheader);	
    }
    $file->append("set out '$pngfn'");
    $file->append("set title 'Select data point to delete'");
    $file2->append("set out '$pngb'");
    $file2->append("set title 'Select data point to delete'");
    $file->append(@adv_opt);
    $file2->append(@adv_opt);
    my @l = $model->print_plot;
    my @l2 = $model->print_bode;
    my $dfile = SemaforeFile->new('/tmp/delete_dat'.$$.'.dat');
    $dfile->writeline($data[$f]);
    $l[-1].= ", '".$dfile->filename."' u 2:(-\$3) w p pt 4 tit 'F'";
    $l2[-1].= ", '".$dfile->filename."' u 1:(abs(\$2)) w p pt 4 tit 'F','' u 1:(abs(\$3)) w p pt 4 tit 'F'";
    $file->append(@l);
    $file2->append(@l2);
    eval {
	system('/usr/bin/gnuplot '.$file->filename);
	system('/usr/bin/gnuplot '.$file2->filename);
    };
    if ($@)
    {
	print "ERROR\n";
        print $@;
	unlink $file->filename if ($file->exist);
	unlink $file2->filename if ($file2->exist);
	unlink $dfile->filename if ($dfile->exist);
	exit;
    }
    unlink $file->filename if ($file->exist);
    unlink $file2->filename if ($file2->exist);
    unlink $dfile->filename if ($dfile->exist);
    print "OK\n";
    if ($plotmode =~ /bode/i)
    {
	print "/analytic/png/bode_delete_".$$.".png\n";
    }
    else
    {
	print "/analytic/png/delete_".$$.".png\n";
    }
    ### Clean old files in pngdir ###
    my $dir = $Impedance::Header::htmldir.'/png/';
    opendir DIR,$dir or die "Could not open directory '$dir' $!\n";
    while (my $file = readdir(DIR))
    {
	next unless ($file =~ /png/);
	my $fn = $dir.$file;
        my $sb = stat($fn);
        next unless ((time-300) > $sb->mtime);
	if ($fn =~ /([\w\.\/]+)/)
        {
            unlink $1 if -e ($1);
        }
    }
    exit;
}

if ($ac eq 'help_fit')
{
    my $m = Impedance::IMPCGI::format_model(param('model'));
    my $model1 = Impedance::Model->new();
    my $nr = $model1->parse($m);
    $model1->data(param('data'));
    my $min = param('min');
    my $max = param('max');
    my $sum = param('sum');
    if ($min <= $max || $sum >= $min || $sum <= $max)
    {
	print "Fitting not possible without a valid frequency range.\nMinfrequency must be less than maxfrequency\nand summit/bend frequency must be inbetween";
	exit;
    }
    my $f = param('f');
    my @d = split(/\n/,$model1->data());
    my $minf = $d[$min];
    my $maxf = $d[$max];
    my $sumf = $d[$sum];
    my $fmin = $minf;
    my $fmax = $maxf;
    foreach ($fmin,$fmax)
    {
	if (/^[\s\t]*([\d\.\+\-eE]+)/)
	{
	    $_ = $1;
	}
    }
    my $model = Impedance::Model->new($model1,$fmin,$fmax);
    $model->scale_factor($scalefn);
    $model->set_xlabel("Z_real(".$slabels{$scale}.")");
    $model->set_ylabel("-Z_imag(".$slabels{$scale}.")");
    $model->set_ylabel_bode("-Z(".$slabels{$scale}.")");
    my @fitlist = ();
    foreach ($model->elements())
    {
	push @fitlist,$model->element($_)->fit;
    }
    my $element = $model->element($nr);
    my $e2 = $model1->element($nr);
    foreach my $tag ($element->helpfunction())
    {
	my $val = $element->helpfunction($tag,$minf,$sumf,$maxf);
	$element->value($tag,$val);
    }
    my $file = SemaforeFile->new('/tmp/model_'.$$.'.gp');
    my $pngfn = $pngdir.'model_'.$$.'.png';
    my $out = SemaforeFile->new('/tmp/fit.log','/tmp/fit.lock');
    my $outf = SemaforeFile->new('/tmp/result_'.$$.'.dat');
    $out->lock_ex();
    unlink $out->filename if (-e $out->filename);
    eval {
        $file->append($model->print_fit(undef,@fitlist));
	$file->append("sa '".$outf->filename."'");
        chdir '/tmp/';
        system('/usr/bin/gnuplot '.$file->filename.' > /dev/null 2> /dev/null');
    };
    if ($@)
    {
	print "ERROR\n";
        print $@;
    }
    my @res = ();
    @res = $outf->readlines() if ($outf->exist);
    print "OK\n" if (@res);
    foreach my $l (@res)
    {
        next unless ($l =~ /^Final/);
        my ($n,$v) = split(/\s*\=\s*/,$l);
	warn "Warning: malformed value detected! :",$l if (!defined($v) || $v !~ /[eE\+\-\d]+/ || $v =~ /[\{\}]/);
	if ($v =~ /[\{\}]/)
	{
	    ### Handle cases where fit results in complex values, only use real part. ###
	    $v =~ s/[\{\}\s]//g;
	    my ($re,$im) = split(/\,/,$v);
	    $v = $re;
	}
	$v = sprintf("%.4g",$v);
        if ($n =~ /Final_(\w+)_(\d+)/)
        {
            my $tag = $1;
            my $nr = $2;
            if ($n =~ /fmax/i)
            {
                print "INFO: Maximum frequency for arc for element ".$model->element($nr)->name.": $v\n";
            }
	    elsif($n =~ /Cps/)
	    {
		print "INFO: Pseudo capacitance for arc for element ".$model->element($nr)->name.": $v\n";
	    }
            else
            {
                my $e = $model->element($nr);
                $e->value($tag,$v);
                print $tag.'_'.$nr.':'.$v."\n";
            }
	    my $e = $model1->element($nr);
	    $e->value($tag,$v);
        }
    }
    @res = ();
    $out->lock_un();
    unlink $out->filename if ($out->exist);
    $file->writeline($Impedance::Header::gpheader);
    $file->append("set out '$pngfn'");
    $file->append("set title 'Residual data not yet accounted for by current model'");
    $file->append(@adv_opt);
    eval {
        $file->append($model1->print_plot);
        system('/usr/bin/gnuplot '.$file->filename);
    };
    if ($@)
    { 
	print "ERROR\n";
        print $@;
    }
    unlink $file->filename if ($file->exist);
    unlink $outf->filename if ($outf->exist);
    print "INFO: \n";
    print "FILE:/analytic/png/model_".$$.".png\n";
    ### Clean old files in pngdir ###
    my $dir = $Impedance::Header::htmldir.'/png/';
    opendir DIR,$dir or die "Could not open directory '$dir' $!\n";
    while (my $file = readdir(DIR))
    {
	next unless ($file =~ /png/);
	my $fn = $dir.$file;
        my $sb = stat($fn);
        next unless ((time-300) > $sb->mtime);
	if ($fn =~ /([\w\.\/]+)/)
        {
            unlink $1 if -e ($1);
        }
    }
    exit;
}

if ($ac eq 'select_points')
{
    my $model = Impedance::Model->new();
    my $m = Impedance::IMPCGI::format_model(param('model'));
    $model->parse($m);
    $model->data(param('data'));
    $model->scale_factor($scalefn);
    $model->set_xlabel("Z_real(".$slabels{$scale}.")");
    $model->set_ylabel("-Z_imag(".$slabels{$scale}.")");
    $model->set_ylabel_bode("-Z(".$slabels{$scale}.")");
    $model->set_admittance_mode($admode) if ($admode);
    my $m2 = Impedance::Model->new();
    $m2->data($model->subtract_model);
    my $min = param('min');
    my $max = param('max');
    my $sum = param('sum');
    my $f = param('f');
    foreach ($min,$sum,$max,$f)
    {
	if (/\s+/)
	{
	    print "Oops, $_ is not a frequency!";
	    exit;
	}
    }
    my @d = split(/\n/,$m2->data());
    for( my $x = 0; $x < @d; $x++)
    {
	next unless ($x == $min);
	$min = $d[$x];
	last;
    }
    for( my $x = 0; $x < @d; $x++)
    {
	next unless ($x == $max);
	$max = $d[$x];
	last;
    }
    for( my $x = 0; $x < @d; $x++)
    {
	next unless ($x == $sum);
	$sum = $d[$x];
	last;
    }
    for( my $x = 0; $x < @d; $x++)
    {
	next unless ($x == $f);
	$f = $d[$x];
	last;
    }
    my $file = SemaforeFile->new('/tmp/model_'.$$.'.gp');
    my $pngfn = $pngdir.'model_'.$$.'.png';
    my $bgcolor = get_CGI_value_clean('bgcolor');
    $file->writeline($Impedance::Header::gpheader);	
    $file->append("set out '$pngfn'");
    $file->append("set title 'Residual data not yet accounted for by current model'");
    my $dfile = SemaforeFile->new('/tmp/help_dat'.$$.'.dat');
    $dfile->writeline(join(' ',$min,$sum,$max,$f));
    my $minr = param('minr');
    my $maxr = param('maxr');
    my @l = $m2->print_plot;
    if (defined($minr) && defined($maxr))
    {
	@l = $m2->print_plot_range($minr,$maxr); 
	unshift @l,"set xrange [$minr:$maxr]";
    }
    $l[-1].= ", '".$dfile->filename."' u 2:(-\$3) w p tit 'min', '' u 5:(- \$6) w p tit 'sum/bend', '' u 8:(-\$9) w p tit 'max', '' u 11:(-\$12) w p tit 'F'";
    $file->append(@l);
    eval {
	system('/usr/bin/gnuplot '.$file->filename);
    };
    if ($@)
    { 
	print "ERROR\n";
        print $@;
    }
    print "OK\n";
    print '/analytic/png/model_'.$$.".png\n";
    unlink $file->filename;
    unlink $dfile->filename;
    ### Clean old files in pngdir #
    my $dir = $Impedance::Header::htmldir.'/png/';
    opendir DIR,$dir or die "Could not open directory '$dir' $!\n";
    while (my $file = readdir(DIR))
    {
	next unless ($file =~ /png/);
	my $fn = $dir.$file;
        my $sb = stat($fn);
        next unless ((time-300) > $sb->mtime);
	if ($fn =~ /([\w\.\/]+)/)
        {
            unlink $1 if -e ($1);
        }
    }
    exit;
}
print "Error, unknown query\n";




sub fit($)
{
    my $filetype = shift;
    my $minf = get_CGI_value('min');
    my $maxf = get_CGI_value('max');
    my $fitmode = get_CGI_value('fitmode');
    my $m = Impedance::IMPCGI::format_model(param('model'));
    my $model1 = Impedance::Model->new();
    my $arcs = get_CGI_value_clean('arcs');
    $model1->parse($m);
    $model1->data(param('data'));
    my $model;
    if ($minf && $maxf)
    {
	print $m;
	$model = Impedance::Model->new($model1,$minf,$maxf);
    }
    else
    {
	$model = $model1;
    }
    $model->set_show_arcs($arcs);
    $model->scale_factor($scalefn);
    $model->set_xlabel("Z_real(".$slabels{$scale}.")");
    $model->set_ylabel("-Z_imag(".$slabels{$scale}.")");
    $model->set_ylabel_bode("-Z(".$slabels{$scale}.")");
    $model->set_admittance_mode($admode) if ($admode);
    my @fitlist = split(/\s*\,\s*/,param('fitlist'));
    my $file = SemaforeFile->new('/tmp/model_'.$$.'.gp');
    my $file2 = SemaforeFile->new('/tmp/bode_'.$$.'.gp');
    my $title = param('filename');
    my $epstit = $title;
    $epstit =~ s/_/\\_/g if ($epstit);
    if ($title)
    {
        $title .= ', '.$model->model_text;
        $epstit .= ', '.$model->model_text;
    }
    else
    {
        $title = $model->model_text;
        $epstit = $model->model_text;
    }
    my $pngfn = $pngdir.'model_'.$$.'.png';
    my $pngb = $pngdir.'bode_'.$$.'.png';
    my $pngrep = $pngdir.'report_model_'.$$.'.eps';
    my $pngbod = $pngdir.'report_bode_'.$$.'.eps';
    my $pngerr = $pngdir.'report_error_'.$$.'.png';
    my $epserr = $pngdir.'report_error_'.$$.'.eps';
    if ($filetype eq 'png')
    {
	$pngrep = $pngdir.'report_model_'.$$.'.png';
	$pngbod = $pngdir.'report_bode_'.$$.'.png';
    }
    my $out = SemaforeFile->new('/tmp/fit.log','/tmp/fit.lock');
    my $outf = SemaforeFile->new('/tmp/result_'.$$.'.dat');
    $out->lock_ex();
    unlink $out->filename if (-e $out->filename);
    eval {
	if ($fitmode && $fitmode =~ /normal/i)
	{
	    $file->append($model->print_fit(undef,@fitlist));
	}
	else
	{
	    $file->append($model->print_fit_weight(undef,@fitlist));
	}
        $file->append("sa '".$outf->filename."'");
        chdir '/tmp/';
        system('/usr/bin/gnuplot '.$file->filename.' > /dev/null 2> /dev/null');
    };
    if ($@)
    {
	print "ERROR\n";
        print $@;
    }
    my @res = ();
    @res = $outf->readlines() if ($outf->exist);
    print "OK\n" if (@res);
    ### Set correct model values ###
    foreach my $l (@res)
    {
        next unless ($l =~ /^Final/);
        my ($n,$v) = split(/\s*\=\s*/,$l);
	if ($n =~ /Final_(\w+)_(\d+)/)
        {
            my $tag = $1;
            my $nr = $2;
	    next if ($n =~ /Cps|fmax/);
	    my $e = $model->element($nr);
	    $e->value($tag,$v);
        }
    }
    if (@res)
    {
	my $rmax = sprintf("%.4g",$model->F(1.0e100)->Re());
	my $rmin = sprintf("%.4g",$model->F(1.0e-100)->Re());
	my $rp = sprintf("%.4g",$rmin - $rmax);
	my ($mean,$max) = $model->get_error();
	print "RES: R(1E100 Hz):  $rmax Ohm (Real part)\n"; 
	print "RES: R(1E-100 Hz):  $rmin Ohm (real part)\n"; 
	print "RES: R(1E100 Hz)-R(1E-100 Hz):  $rp Ohm (Real part)\n"; 
	my $mtext = $model->model_text();
	print "FITERROR: Model: $mtext\n";
	printf("FITERROR: Mean_error: %.2f %%\n",$mean);
	printf("FITERROR: Max_error: %.2f %%\n",$max);
    }
    my @maxfreqs = ();
    my @fvals = ();
    my @pscs = ();
    foreach my $l (@res)
    {
        next unless ($l =~ /^Final/);
        my ($n,$v) = split(/\s*\=\s*/,$l);
	$v = sprintf("%.4g",$v);
	if ($n =~ /Final_(\w+)_(\d+)/)
        {
            my $tag = $1;
            my $nr = $2;
            if ($n =~ /fmax/i)
            {
		push @maxfreqs,$model->element($nr)->name.":$v";
                print "INFO: Maximum frequency for arc for element ".$model->element($nr)->name.": $v\n";
            }
	    elsif($n =~ /Cps/i)
	    {
		push @pscs,$model->element($nr)->name.":$v";
                print "INFO: Pseudo capacitance for arc for element ".$model->element($nr)->name.": $v\n";
	    }
            else
            {
                my $e = $model->element($nr);
                $e->value($tag,$v);
		push @fvals,$tag.'_'.$nr.":$v";
                print $tag.'_'.$nr.':'.$v."\n";
            }
        }
    }
    print "FITDATA\n";
    @res = ();
    my $found = 0;
    my @lines = $out->readlines();
    $out->lock_un();
    unlink $out->filename if ($out->exist);
    my $bgcolor = get_CGI_value_clean('bgcolor');
    if ($bgcolor && $bgcolor =~ /white/i)
    {
	$file->writeline($Impedance::Header::gpheader_white);	
	$file2->writeline($Impedance::Header::gpheader_white);	
    }
    else
    {
	$file->writeline($Impedance::Header::gpheader);	
	$file2->writeline($Impedance::Header::gpheader);	
    }
    foreach my $tag ('xmin','xmax','ymin','ymax')
    {
        my $val = get_CGI_value_clean($tag);
	next unless (defined($val) && $val =~ /\d/);
        $model->set_limit($tag,$val);
    }
    $file->append("set out '$pngfn'");
    $file2->append("set out '$pngb'");
    eval {
	$file->append("set title '$title'");
	$file->append('set xzeroaxis lt -1');
	$file->append('set yzeroaxis lt -1');
	$file->append(@adv_opt);
        $file->append($model->print_plot);
	$file2->append("set title '$title'");
	$file2->append(@adv_opt);
        $file2->append($model->print_bode);
        system('/usr/bin/gnuplot '.$file->filename);
        system('/usr/bin/gnuplot '.$file2->filename);
	if ($filetype eq 'png')
	{
	    $file->writeline('set term png');
	    $file->append('set xzeroaxis lt -1');
	    $file->append('set yzeroaxis lt -1');
	    $file->append('set size 1,0.8');
	    $file->append("set title '$title'");
	    $file->append("set out '$pngrep'");
	    $file->append($model->print_plot);
	    $file2->writeline('set term png');
	    $file2->append('set size 1,0.8');
	    $file2->append("set title '$title'");
	    $file2->append("set out '$pngbod'");
	    $file2->append($model->print_bode);
	}
	else
	{
	    $model->set_xlabel("Z_{real}(".$slabelseps{$scale}.")");
	    $model->set_ylabel("-Z_{imag}(".$slabelseps{$scale}.")");
	    $model->set_ylabel_bode("-Z(".$slabelseps{$scale}.")");
	    $file->writeline('set term postscript enhanced eps');
	    $file->append('set xzeroaxis lt -1');
	    $file->append('set yzeroaxis lt -1');
	    $file->append('set size 1,0.8');
	    $file->append("set title '$epstit'");
	    $file->append("set out '$pngrep'");
	    $file->append(@adv_opt);
	    $file->append($model->print_plot_eps);
	    $file2->writeline('set term postscript enhanced eps');
	    $file2->append('set size 1,0.8');
	    $file2->append("set title '$epstit'");
	    $file2->append("set out '$pngbod'");
	    $file2->append($model->print_bode_eps);
	}
        system('/usr/bin/gnuplot '.$file->filename);
        system('/usr/bin/gnuplot '.$file2->filename);
	$file->writeline($Impedance::Header::gpheader);
	$file->append('set size 1,0.6');
	$file->append("set title '$title'");
	$file->append("set out '$pngerr'");
	$file->append('set xzeroaxis lt -1');
	$file->append($model->print_plot_error);
        system('/usr/bin/gnuplot '.$file->filename);
	unless ($filetype eq 'png')
	{
	    $file->writeline('set term postscript enhanced eps');
	    $file->append('set size 1,0.6');
	    $file->append("set title '$epstit'");
	    $file->append("set out '$epserr'");
	    $file->append('set xzeroaxis lt -1');
	    $file->append($model->print_plot_error);
	    system('/usr/bin/gnuplot '.$file->filename);
	}
        system('/usr/bin/gnuplot '.$file2->filename);
    };
    if ($@)
    { 
	print "ERROR\n";
        print $@;
    }
    unlink $file->filename if ($file->exist);
    unlink $file2->filename if ($file2->exist);
    unlink $outf->filename if ($outf->exist);
    print join("\n",@lines),"\n";
    print "END_TEXT\n";
    print "FILE:model_".$$.".png\n";
    my @flst = ();;
    if ($filetype eq 'png')
    {
	print "REPORT:report_model_".$$.".png\n";
	print "ERROR:report_error_".$$.".png\n";
	push @flst,$pngdir."report_model_".$$.".png";
	push @flst,$pngdir."report_error_".$$.".png";
    }
    else
    {
	print "REPORT:report_model_".$$.".eps\n";
	print "ERROR:report_error_".$$.".eps\n";
	push @flst,$pngdir."report_model_".$$.".eps";
	push @flst,$pngdir."report_error_".$$.".eps";
    }
    print "POP:report_error_".$$.".png\n";
    push @flst,join("\n",@lines);
    ### Clean old files in pngdir ###
    my $dir = $Impedance::Header::htmldir.'/png/';
    opendir DIR,$dir or die "Could not open directory '$dir' $!\n";
    while (my $file = readdir(DIR))
    {
	next unless ($file =~ /png|eps|pdf/);
	my $fn = $dir.$file;
        my $sb = stat($fn);
        next unless ((time-300) > $sb->mtime);
	if ($fn =~ /([\w\.\/]+)/)
        {
            unlink $1 if -e ($1);
        }
    }
    return @flst,\@maxfreqs,\@fvals,\@pscs;
}
