#!/usr/bin/perl
use strict;
use warnings;
use SemaforeFile;
my $dir = '../Impedance/';
my @list = ();
if (@ARGV)
{
    while (@ARGV)
    {
	my $file = shift;
	push @list,$file;
    }
}
else
{
    @list = ('../libs/Debug.pm','../libs/SemaforeFile.pm','../libs/SocketClient.pm');
    foreach ('Header.pm','IMPCGI.pm','Base.pm','RQ.pm','W.pm','Complex.pm','Device.pm','Model.pm')
    {
	push @list,$dir.$_;
    }
}
my $row_l = '';
my $row_r = '';
foreach my $file (@list)
{
    my $f = SemaforeFile->new($file);
    warn $f->filename(),"\n";
    $f->open_read();
    my @data = ();
    my $inherit = '';
  INNER:
    while (my $str = $f->readline())
    {
	chomp $str;
	if ($str =~ /\@ISA\s*\=\s*qw\(([\w\:]+)\)/)
	{
	    $inherit = $1;
	}
	last INNER if ($str =~ /\#{10}/);
    }
  INNER2:
    while (my $str = $f->readline())
    {
	last INNER2 if ($str =~ /\#{5}/);
	chomp $str;
	$str =~ s/^\#+[\s\t]*//;
	push @data,$str;
    }
    unless ($inherit)
    {
	while (my $str = $f->readline())
	{
	    last if ($str =~ /BEGIN/);
	    if ($str =~ /\@ISA\s*\=\s*qw\(([\w\:]+)\)/)
	    {
		$inherit = $1;
		last;
	    }
	}
    }
    ### Print latex code ###
    my $tab = 0;
  INNER3:
    for (my $x = 0; $x < @data; $x++)
    {
	if (!$tab && $data[$x] =~ /^[\s\t]*package[\s\t]+(\w+\:*\:*\w+)[\s\t\;]*$/i)
	{
	    print '\section{'.$1."}\n";
	    print '\label{module_'.$1."}\n";
	    if ($inherit)
	    {
		print "Inherits from $inherit (refer section \\ref{module_$inherit}).\n\n";
	    }
	    next;
	}
	if (!$tab && $data[$x] =~ /^[\s\t]*(\w+\:\:\w+)[\s\t\;]*$/i)
	{
	    print '\section{'.$1."}\n";
	    print '\label{module_'.$1."}\n";
	    if ($inherit)
	    {
		print "Inherits from $inherit (refer section \\ref{module_$inherit}).\n\n";
	    }
	    next;
	}
	if ($data[$x] =~ /^[\s\t]*$/)
	{
	    if ($x == @data-1)
	    {
		&tab_end() if ($tab);
		&print_line($data[$x]);
		last INNER3;
	    }
	    if ($tab)
	    {
		$tab = 0;
		&tab_end();
		&print_line($data[$x]);
		next INNER3;
	    }
	    else
	    {
		if ($data[$x+1] =~ /\-\>/ && $data[$x+1] !~ /\-\>new/)
		{
		    $tab = 1;
		    print "\n";
		    &print_line($data[$x]);
		    &tab_head();
		    next INNER3;
		}
	    }
	    &print_line($data[$x]);
	}
	else
	{
	    if ($tab)
	    {
		my @l = ();
		if ($data[$x] =~ /\-\>/)
		{
		    if ($data[$x] =~ /(^[\$\w\-\>\(\)\$\@\%\,\[\]]+)[\s\t]+(.*)/)
		    {
			&tab_row($1,$2);
		    }
		    else
		    {
			&tab_row($data[$x],'');
		    }
		}
		else
		{
		    &tab_row('',$data[$x]);
		}
	    }
	    else
	    {
		&print_line($data[$x]);
	    }
	}
    }
    print "\n\n";
}


sub tab_head()
{
    print '\begin{longtable}{ l p{9.5cm} }',"\n";
    $row_l = '';
    $row_r = '';
}

sub tab_row
{
    my @l = @_;
    foreach (@l)
    {
	chomp;
	s/\\/\\backslash/g;
	s/\$/\\\$/g;
	s/\-\>/\$\\rightarrow\$/g;
	s/_/\\_/g;
	s/\\backslash/\$\\backslash\$/g;
	s/\#/\\#/g;
    }
    if ($row_l && $l[0])
    {
	print join(' & ',$row_l,$row_r),' \\\\',"\n";
	$row_l = $l[0];
	$row_r = '';	
    }
    $row_l = $l[0] if ($l[0]);
    $row_r .= ' '.$l[1];
}

sub tab_end()
{
    if ($row_l || $row_r)
    {
	print join(' & ',$row_l,$row_r),' \\\\',"\n";
	$row_l = '';
	$row_r = '';
    }
    print '\end{longtable}',"\n";
}

sub print_line($)
{
    my $l = shift;
    chomp $l;
    if ($l =~ /\-{2}[\s\t]*(.*)/)
    {
	$l = $1;
	$l =~ s/\$/\\\$/g;
	$l =~ s/\-\>/\$\\rightarrow\$/g;
	$l =~ s/_/\\_/g;
	$l =~ s/\\backslash/\$\\backslash\$/g;
	$l =~ s/\#/\\#/g;
	print $l,"\n";
	return;
    }
    $l =~ s/\\/\\backslash/g;
    $l =~ s/\$/\\\$/g;
    $l =~ s/\-\>/\$\\rightarrow\$/g;
    $l =~ s/_/\\_/g;
    $l =~ s/\\backslash/\$\\backslash\$/g;
    $l =~ s/\#/\\#/g;
    print $l,"\n";
}
